/**
  * Bouton plat et transparent, developpé initialement pour le projet
  * Imageo (le-terrier d'AbulEdu) et repris dans les classes de base
  * d'AbulÉdu (merci Thomas !)
  *
  * @see https://redmine.ryxeo.com/projects/leterrier-imageo
  * @author 2011 Thomas Grosjean-Pasut <thomas.grosjeanpasut@ryxeo.com>
  * @author 2011 Eric Seigne <eric.seigne@ryxeo.com>
  * @author 2011 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  * @see The GNU Public License (GPL)
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "abuleduflatboutonv1.h"
#include <QMouseEvent>
#include <QDebug>

AbulEduFlatBoutonV1::AbulEduFlatBoutonV1(QWidget* parent) :
   QPushButton(parent)
{
    setFocusPolicy(Qt::NoFocus);
    m_couleursCSS = new QMap<QString, QString>();
    m_icones = new QMap<QString, QString>();

// Couleurs par défaut
    setCouleursTexte(QColor(Qt::black), QColor(Qt::blue), QColor(Qt::blue), QColor(Qt::gray));
    setCouleursFond(QColor(Qt::transparent), QColor(Qt::transparent), QColor(Qt::transparent), QColor(Qt::transparent));

// Pas d'icones par défaut
    setIcones("","","");
}



void AbulEduFlatBoutonV1::setCouleursTexte(QColor normale, QColor survol, QColor pressed, QColor disabled)
{
    if(normale.isValid())
    {
            setCouleurTexteNormale(normale);
    }
    if(survol.isValid())
    {
        setCouleurTexteSurvol(survol);
    }
    if(pressed.isValid())
    {
        setCouleurTextePressed(pressed);
    }
    if(disabled.isValid())
    {
        setCouleurTexteDisabled(disabled);
    }
}

void AbulEduFlatBoutonV1::setCouleurTexteNormale(QColor couleur)
{
    m_couleursCSS->insert("couleurTexteNormale", toRGBA(couleur));
    updateCSS();
}

void AbulEduFlatBoutonV1::setCouleurTexteSurvol(QColor couleur)
{
    m_couleursCSS->insert("couleurTexteSurvol", toRGBA(couleur));
    updateCSS();
}

void AbulEduFlatBoutonV1::setCouleurTextePressed(QColor couleur)
{
    m_couleursCSS->insert("couleurTextePressed", toRGBA(couleur));
    updateCSS();
}

void AbulEduFlatBoutonV1::setCouleurTexteDisabled(QColor couleur)
{
    m_couleursCSS->insert("couleurTexteDisabled", toRGBA(couleur));
    updateCSS();
}

void AbulEduFlatBoutonV1::setCouleursFond(QColor normale, QColor survol, QColor pressed, QColor disabled)
{
    if(normale.isValid())
    {
        setCouleurFondNormale(normale);
    }
    if(survol.isValid())
    {
        setCouleurFondSurvol(survol);
    }
    if(pressed.isValid())
    {
        setCouleurFondPressed(pressed);
    }
    if(disabled.isValid())
    {
        setCouleurFondDisabled(disabled);
    }
}

void AbulEduFlatBoutonV1::setCouleurFondNormale(QColor couleur)
{
    m_couleursCSS->insert("couleurFondNormale", toRGBA(couleur));
    updateCSS();
}

void AbulEduFlatBoutonV1::setCouleurFondSurvol(QColor couleur)
{
    m_couleursCSS->insert("couleurFondSurvol", toRGBA(couleur));
    updateCSS();
}

void AbulEduFlatBoutonV1::setCouleurFondPressed(QColor couleur)
{
    m_couleursCSS->insert("couleurFondPressed", toRGBA(couleur));
    updateCSS();
}

void AbulEduFlatBoutonV1::setCouleurFondDisabled(QColor couleur)
{
    m_couleursCSS->insert("couleurFondDisabled", toRGBA(couleur));
    updateCSS();
}
void AbulEduFlatBoutonV1::setIcones(QString normale, QString survol, QString pressed, QString disabled)
{

    if(QFile(normale).exists())
        m_icones->insert("iconeNormale", normale);
    else
        m_icones->insert("iconeNormale", "");

    if(QFile(survol).exists())
        m_icones->insert("iconeSurvol", survol);
    else
    m_icones->insert("iconeSurvol", "");

    if(QFile(pressed).exists())
        m_icones->insert("iconePressed", pressed);
    else
        m_icones->insert("iconePressed", "");

    if(QFile(disabled).exists())
        m_icones->insert("iconeDisabled", disabled);
    else
        m_icones->insert("iconeDisabled", "");

    updateCSS();
}

void AbulEduFlatBoutonV1::setIconeNormale(QString normale)
{
    if(QFile(normale).exists())
        m_icones->insert("iconeNormale", normale);
    else
        m_icones->insert("iconeNormale", "");
}

void AbulEduFlatBoutonV1::setIconeSurvol(QString survol)
{
    if(QFile(survol).exists())
        m_icones->insert("iconeSurvol", survol);
    else
    m_icones->insert("iconeSurvol", "");
}

void AbulEduFlatBoutonV1::setIconePressed(QString pressed)
{
    if(QFile(pressed).exists())
        m_icones->insert("iconePressed", pressed);
    else
        m_icones->insert("iconePressed", "");
}

void AbulEduFlatBoutonV1::setIconeDisabled(QString disabled)
{
    if(QFile(disabled).exists())
        m_icones->insert("iconeDisabled", disabled);
    else
        m_icones->insert("iconeDisabled", "");
}

QString AbulEduFlatBoutonV1::toRGBA(QColor couleur)
{
    QString s="";
    if(couleur.isValid())
    {
        s.append("rgba(");
        s.append(QString::number(couleur.red()));
        s.append(",");
        s.append(QString::number(couleur.green()));
        s.append(",");
        s.append(QString::number(couleur.blue()));
        s.append(",");
        s.append(QString::number(couleur.alpha()));
        s.append(")");
    }
    else
    {
        s="(0,0,0,0)";
    }
    return s;
}

void AbulEduFlatBoutonV1::updateCSS()
{
    m_cssFlatBouton.clear();
    // Etat normal du bouton------------------------------------------------------------------
    m_cssFlatBouton.append(QString("*{border: none; color:"+m_couleursCSS->value("couleurTexteNormale")+";"));
    // Ajout d'une couleur de fond
    m_cssFlatBouton.append(QString("background-color:"+m_couleursCSS->value("couleurFondNormale")+";"));
    // Ajout d'une icone s'il y en a une
    if(m_icones->value("iconeNormale").size() > 0)
       {
           m_cssFlatBouton.append(QString("image:url("+m_icones->value("iconeNormale")+");"));
       }
    m_cssFlatBouton.append(QString("image-position: left;}"));

    // Etat normal du survolé du bouton-------------------------------------------------------
    m_cssFlatBouton.append(QString(":hover{color:"+m_couleursCSS->value("couleurTexteSurvol")+";"));
    // Ajout d'une couleur de fond
    m_cssFlatBouton.append(QString("background-color:"+m_couleursCSS->value("couleurFondSurvol")+";"));
    // Ajout d'une icone s'il y en a une
    if(m_icones->value("iconeSurvol").size() > 0)
       {
           m_cssFlatBouton.append(QString("image:url("+m_icones->value("iconeSurvol")+");"));
       }
    m_cssFlatBouton.append(QString("image-position: left;}"));

    // Etat pressé du bouton----------------------------------------------------------------
    m_cssFlatBouton.append(QString(":pressed{color:"+m_couleursCSS->value("couleurTextePressed")+";"));
    // Ajout d'une couleur de fond
    m_cssFlatBouton.append(QString("background-color:"+m_couleursCSS->value("couleurFondPressed")+";"));
    // Ajout d'une icone s'il y en a une
    if(m_icones->value("iconePressed").size() > 0)
       {
           m_cssFlatBouton.append(QString("image:url("+m_icones->value("iconePressed")+");"));
       }
    m_cssFlatBouton.append(QString("image-position: left;}"));

    // Etat disabled du bouton------------------------------------------------------------------
    m_cssFlatBouton.append(QString(":disabled{border: none; color:"+m_couleursCSS->value("couleurTexteDisabled")+";"));
    // Ajout d'une couleur de fond
    m_cssFlatBouton.append(QString("background-color:"+m_couleursCSS->value("couleurFondDisabled")+";"));
    // Ajout d'une icone s'il y en a une
    if(m_icones->value("iconeDisabled").size() > 0)
    {
        m_cssFlatBouton.append(QString("image:url("+m_icones->value("iconeDisabled")+");"));
    }
    else
    {
        m_cssFlatBouton.append(QString("image:none")+";"); // Plus d'icône
    }
    m_cssFlatBouton.append(QString("image-position: left;}"));
    setStyleSheet(m_cssFlatBouton);
}

void AbulEduFlatBoutonV1::setCSSFlatBouton(QString css)
{
    m_cssFlatBouton = css; // Attention cette valeur sera écrasée par tout changement dans la feuille de style
    setStyleSheet(m_cssFlatBouton);
}
